@@ -2,7 +2,7 @@ import abc
 import json
 import logging
 import os
-from typing import Dict, Any, Callable, Tuple, List, Generator
+from typing import Dict, Any, Callable, Tuple, List, Generator, Union
 
 from google import auth
 from google.auth import iam
@@ -11,7 +11,7 @@ from google.auth.impersonated_credentials import Credentials as ImpersonatedCred
 from google.auth.transport import requests
 from google.oauth2.service_account import Credentials as ServiceAccountCredentials
 from google.protobuf.duration_pb2 import Duration
-from googleapiclient.discovery import build
+from googleapiclient.discovery import build, Resource
 from googleapiclient.errors import HttpError
 
 from gcp_pilot import exceptions
@@ -38,7 +38,11 @@ def _get_project_default_location(credentials, project_id: str, default_zone: st
     if not location:
         service = build(serviceName='appengine', version='v1', credentials=credentials, cache_discovery=False)
         data = service.apps().get(appsId=project_id).execute()
-        location = data['locationId'] + default_zone
+        location = data['locationId']
+        try:
+            int(location[-1])
+        except ValueError:
+            location = data['locationId'] + default_zone
         _CACHED_LOCATIONS[project_id] = location
     return location
 
@@ -71,7 +75,10 @@ class GoogleCloudPilotAPI(abc.ABC):
         self.project_id = self._set_project_id(project_id=project_id, credential_project_id=credential_project_id)
         self.location = self._set_location(location=location)
 
-        self.client = (self._client_class or build)(
+        self.client = self._build_client(**kwargs)
+
+    def _build_client(self, **kwargs) -> Union[Resource, _client_class]:
+        return (self._client_class or build)(
             credentials=self.credentials,
             **kwargs
         )
@@ -207,18 +214,20 @@ class AccountManagerMixin:
         return f'{prefix}:{email}'
 
     def bind_email_to_policy(self, email: str, role: str, policy: Dict) -> Dict:
+        new_policy = policy.copy()
+
         role_id = role if (role.startswith('organizations/') or role.startswith('roles/')) else f'roles/{role}'
         member = self.as_member(email=email)
 
         try:
-            binding = next(b for b in policy['bindings'] if b['role'] == role_id)
+            binding = next(b for b in new_policy['bindings'] if b['role'] == role_id)
             if member in binding['members']:
-                return policy
+                return new_policy
             binding['members'].append(member)
         except (StopIteration, KeyError):
             binding = {'role': role_id, 'members': [member]}
-            policy['bindings'] = policy.get('bindings', []).append(binding)
-        return policy
+            new_policy.get('bindings', []).append(binding)
+        return new_policy
 
     def unbind_email_from_policy(self, email: str, role: str, policy: Dict):
         role_id = f'roles/{role}'
@@ -261,6 +270,8 @@ def friendly_http_error(func):
     }
     _statuses = {
         'INVALID_ARGUMENT': exceptions.ValidationError,
+        'PERMISSION_DENIED': exceptions.NotAllowed,
+        'NOT_FOUND': exceptions.NotFound,
     }
 
     def inner_function(*args, **kwargs):
@@ -274,7 +285,7 @@ def friendly_http_error(func):
                 details = ''  # TODO: add more details to the exceptions
             else:
                 exception_klass = _statuses.get(errors['status'], None)
-                details = errors['message']
+                details = f"{errors['code']}: {errors['message']}"
 
             if exception_klass:
                 raise exception_klass(details) from exc
@@ -288,6 +299,19 @@ class DiscoveryMixin:
     def _execute(self, method: Callable, **kwargs) -> ResourceType:
         return method(**kwargs).execute()
 
+    def _list(
+            self,
+            method: Callable,
+            result_key: str = 'items',
+            params: Dict[str, Any] = None,
+    ) -> Generator[ResourceType, None, None]:
+        results = self._execute(
+            method=method,
+            **params,
+        )
+        for item in results.get(result_key, []):
+            yield item
+
     def _paginate(
             self,
             method: Callable,
