@@ -1,11 +1,12 @@
 from datetime import datetime
-from typing import Any
+from typing import Any, Dict, Union
 
 from google.cloud import bigquery
 from google.cloud.bigquery import Table, DatasetReference
 
 from gcp_pilot import exceptions
 from gcp_pilot.base import GoogleCloudPilotAPI
+from gcp_pilot.storage import CloudStorage
 
 
 class BigQuery(GoogleCloudPilotAPI):
@@ -17,59 +18,14 @@ class BigQuery(GoogleCloudPilotAPI):
             project=project_id or self.project_id,
         )
 
-    def _to_sql_format(self, value: Any) -> Any:
-        if isinstance(value, str):
-            return f"'{value}'"
-        elif isinstance(value, datetime):
-            return f"'{self.date_to_str(value)}'"
-        return value
-
-    def _get_type(self, variable):
-        TYPES = {
-            'int': 'INT64',
-            'str': 'STRING',
-            'datetime': 'DATETIME',
-            'date': 'DATE',
-            'bool': 'BOOL',
-            'float': 'FLOAT64',
-            'Decimal': 'FLOAT64',
-        }
-
-        python_type = type(variable).__name__
-        param_type = TYPES.get(python_type, None)
-        if param_type is None:
-            raise Exception('Parameter with type {param_type} not supported')
-        return param_type
-
-    def _get_value(self, variable):
-        TYPES = {
-            'Decimal': float,
-        }
-        python_type = type(variable).__name__
-        param_type = TYPES.get(python_type, None)
-        if param_type is not None:
-            return param_type(variable)
-        return variable
-
-    def _create_param(self, key, value):
-        if isinstance(value, list):
-            param_type = self._get_type(variable=value[0])
-            param_class = bigquery.ArrayQueryParameter
-        else:
-            param_type = self._get_type(variable=value)
-            param_class = bigquery.ScalarQueryParameter
-        return param_class(key, param_type, self._get_value(value))
-
-    def execute(
+    async def execute(
             self,
-            sql,
-            params=None,
-            legacy_sql=False,
-            job_id=None,
-            destination_table_name=None,
-            destination_dataset_name=None,
-            destination_project=None,
-            write_disposition=None
+            sql: str,
+            params: Dict[str, Any] = None,
+            destination_table_name: str = None,
+            destination_dataset_name: str = None,
+            destination_project: str = None,
+            truncate: bool = None
     ):
         job_config = bigquery.QueryJobConfig()
         if destination_table_name or destination_dataset_name:
@@ -81,19 +37,21 @@ class BigQuery(GoogleCloudPilotAPI):
             destination_dataset = destination_dataset_name
             destination = f'{destination_project}.{destination_dataset}.{destination_table_name}'
             job_config.destination = destination
-            job_config.write_disposition = write_disposition or 'WRITE_EMPTY'
+            if truncate:
+                job_config.write_disposition = bigquery.WriteDisposition.WRITE_TRUNCATE
+            elif truncate is False:
+                job_config.write_disposition = bigquery.WriteDisposition.WRITE_APPEND
 
         if params:
             query_params = [
-                self._create_param(key, value) for key, value in params.items()
+                _BigQueryParam.parse(key, value) for key, value in params.items()
             ]
             job_config.query_parameters = query_params
-        job_config.use_legacy_sql = legacy_sql
 
-        query_job = self.client.query(sql, job_id=job_id, job_config=job_config)
+        query_job = self.client.query(sql, job_config=job_config)
         return self._wait_for_job(job=query_job)
 
-    def insert_rows(self, dataset_name: str, table_name: str, rows, project_id: str = None):
+    async def insert_rows(self, dataset_name: str, table_name: str, rows, project_id: str = None):
         table = self.get_table(
             table_name=table_name,
             project_id=project_id,
@@ -103,21 +61,22 @@ class BigQuery(GoogleCloudPilotAPI):
         if errors:
             raise Exception(f'Bigquery insert error: {errors}')
 
-    def get_table(self, table_name, project_id=None, dataset_name=None) -> Table:
+    async def get_table(self, table_name: str, project_id: str = None, dataset_name: str = None) -> Table:
         dataset_ref = self._dataset_ref(project_id=project_id, dataset_name=dataset_name)
-        table_ref = dataset_ref.table(table_name)
+        table_ref = dataset_ref.table(table_id=table_name)
         return self.client.get_table(table_ref)
 
-    def load(
+    async def load(
             self,
-            table_name,
-            gcs_file,
-            project_id=None,
-            dataset_name=None,
+            table_name: str,
+            filename: str,
+            project_id: str = None,
+            dataset_name: str = None,
             schema=None,
-            sync=False,
-            truncate=None,
-    ):
+            wait: bool = False,
+            truncate: bool = None,
+            gcs_bucket: str = None,
+    ) -> None:
         job_config = bigquery.LoadJobConfig()
         if schema:
             job_config.schema = schema
@@ -125,7 +84,7 @@ class BigQuery(GoogleCloudPilotAPI):
             job_config.autodetect = True
             job_config._properties['load']['schemaUpdateOptions'] = ['ALLOW_FIELD_ADDITION']
 
-        extension = gcs_file.split('.')[-1]
+        extension = filename.split('.')[-1]
         if extension == 'json':
             source_format = bigquery.job.SourceFormat.NEWLINE_DELIMITED_JSON
         elif extension == 'csv':
@@ -145,24 +104,46 @@ class BigQuery(GoogleCloudPilotAPI):
             dataset_name=dataset_name,
         )
         target_ref = dataset_ref.table(table_name)
-        load_job = self.client.load_table_from_uri(
-            source_uris=gcs_file,
-            destination=target_ref,
-            job_config=job_config,
-        )
 
-        if sync:
+        is_gcs = filename.startswith('gs://')
+        if not is_gcs:
+            if gcs_bucket:
+                gcs = CloudStorage(project_id=self.project_id)
+                blob = await gcs.upload(
+                    source_file=filename,
+                    bucket_name=gcs_bucket,
+                )
+                file_url = gcs.get_uri(blob=blob)
+
+                load_job = self.client.load_table_from_uri(
+                    source_uris=file_url,
+                    destination=target_ref,
+                    job_config=job_config,
+                )
+            else:
+                load_job = self.client.load_table_from_file(
+                    file_obj=open(filename, 'rb'),
+                    destination=target_ref,
+                    job_config=job_config,
+                )
+        else:
+            load_job = self.client.load_table_from_uri(
+                source_uris=filename,
+                destination=target_ref,
+                job_config=job_config,
+            )
+
+        if wait:
             self._wait_for_job(job=load_job)
 
-    def copy(
+    async def copy(
             self,
-            source_table_name,
-            destination_table_name,
-            destination_dataset_name,
-            destination_project=None,
-            sync=False,
-            **kwargs,
-    ):
+            source_table_name: str,
+            destination_table_name: str,
+            destination_dataset_name: str,
+            destination_project: str = None,
+            wait: bool = False,
+    ) -> None:
         source_ref = self.get_table(table_name=source_table_name)
         dataset_ref = self._dataset_ref(
             project_id=destination_project or self.project_id,
@@ -170,8 +151,8 @@ class BigQuery(GoogleCloudPilotAPI):
         )
         target_ref = dataset_ref.table(destination_table_name)
 
-        job = self.client.copy_table(source_ref, target_ref, **kwargs)
-        if sync:
+        job = self.client.copy_table(sources=source_ref, destination=target_ref)
+        if wait:
             self._wait_for_job(job=job)
 
     @classmethod
@@ -189,3 +170,69 @@ class BigQuery(GoogleCloudPilotAPI):
             return job.result()
         except Exception as e:
             raise exceptions.BigQueryJobError(job) from e
+
+    def add_external_gcs_source(
+            self,
+            gcs_url: str,
+            dataset_name: str,
+            table_name: str,
+            skip_rows: int = 0,
+            delimiter: str = ',',
+            quote: str = '\"',
+            source_format: str = 'CSV',
+            project_id: str = None
+    ):
+        dataset_ref = self._dataset_ref(dataset_name=dataset_name, project_id=project_id)
+        table = dataset_ref.table(table_name)
+
+        external_config = bigquery.ExternalConfig(source_format=source_format)
+        external_config.autodetect = True
+        external_config.source_uris = [gcs_url]
+        external_config.options.skip_leading_rows = skip_rows
+        external_config.options.field_delimiter = delimiter
+        external_config.options.quote = quote
+
+        table.external_data_configuration = external_config
+
+        return self.client.create_table(table)
+
+
+class _BigQueryParam:
+    @classmethod
+    def _get_type(cls, variable: Any) -> str:
+        TYPES = {
+            'int': 'INT64',
+            'str': 'STRING',
+            'datetime': 'DATETIME',
+            'date': 'DATE',
+            'bool': 'BOOL',
+            'float': 'FLOAT64',
+            'Decimal': 'FLOAT64',
+        }
+
+        python_type = type(variable).__name__
+        param_type = TYPES.get(python_type, None)
+        if param_type is None:
+            raise exceptions.ValidationError(f'Parameter with type {param_type} not supported')
+        return param_type
+
+    @classmethod
+    def _get_value(cls, variable: Any) -> Any:
+        TYPES = {
+            'Decimal': float,
+        }
+        python_type = type(variable).__name__
+        param_type = TYPES.get(python_type, None)
+        if param_type is not None:
+            return param_type(variable)
+        return variable
+
+    @classmethod
+    def parse(cls, key: str, value: Any) -> Union[bigquery.ArrayQueryParameter, bigquery.ScalarQueryParameter]:
+        if isinstance(value, list):
+            param_type = cls._get_type(variable=value[0])
+            param_class = bigquery.ArrayQueryParameter
+        else:
+            param_type = cls._get_type(variable=value)
+            param_class = bigquery.ScalarQueryParameter
+        return param_class(key, param_type, cls._get_value(value))
